#!/usr/bin/env python
# -*- encoding: utf-8; py-indent-offset: 4 -*-

#
# 31.07.2017 : Th.L. : Cisco software suggestion API Framework
#
#
#  https://developer.cisco.com/docs/support-apis/
#
import logging
import os
import json
import time
import ConfigParser
import ciscosupport
import ciscoapi
import sys


# compact suggestions by pid (create dictionary with pids as key and list of product/suggestions value)
def compact_sugesstions_by_pid(suggestions):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    suggestions_by_pid = {}
    pids_notfound = []
    for response in suggestions:
        if response is not None:
            for product in response:
                if product.get('product').get('productName').replace(' ', '') == '':
                    pids_notfound.append(product.get('product').get('basePID'))
                else:
                    pid = product.get('product').get('basePID')
                    temp_product = product.get('product')
                    temp_product.pop('basePID')

                    suggestion = product.get('suggestions', None)
                    if suggestions_by_pid.get(pid, None) is not None:
                        suggestions_by_pid.get(pid).append({'suggestion': suggestion, 'product': temp_product})
                    else:
                        suggestions_by_pid.update({pid: [{'suggestion': suggestion, 'product': temp_product}]})
    return [suggestions_by_pid, pids_notfound]


# save suggestion records to file by PID,
# expects a list of dictionary from compact_sugesstions_by_pid,
# returns a list of saved PIDs
def save_suggested(suggestions_by_pid, path):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    saved = []
    pids = suggestions_by_pid.keys()

    for pid in pids:
        suggestion = suggestions_by_pid.get(pid)
        pidfile = pid.replace('/', '_')
        with open(path + pidfile, 'w') as f:
            json.dump(suggestion, f)
            saved.append(pid)

    return saved


def save_pids_notfound(pids, path):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    saved = []

    for pid in pids:
        pidfile = pid.replace('/', '_')
        with open(path + pidfile, 'w+') as f:
            pass
        saved.append(pid)

    return saved


def main():

    ciscosupport.set_logging('debug')
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    access_token = ''
    lifetime = 0
    pids = []

    conf_file = '~/etc/ciscoapi/ciscoapi.conf'
    conf_file = os.path.expanduser(conf_file)

    # default paths
    base_path = '~/var/ciscoapi'

    # refresh time in days
    suggestion_refresh_found = 31
    suggestion_refresh_notfound = 7

    wait_after_start = True
    max_wait_time = 15
    loglevel = 'warning'

    if os.path.isfile(conf_file):
        configParser = ConfigParser.RawConfigParser()
        configParser.read(conf_file)
        if configParser.has_option('global', 'base_path'):
            wait_after_start = configParser.getboolean('global', 'base_path')
        if configParser.has_option('global', 'wait_after_start'):
            wait_after_start = configParser.getboolean('global', 'wait_after_start')
        if configParser.has_option('global', 'max_wait_time'):
            max_wait_time = configParser.getint('global', 'max_wait_time')
        if configParser.has_option('global', 'loglevel'):
            loglevel = configParser.get('global', 'loglevel')
        if configParser.has_option('suggestion', 'suggestion_refresh_found'):
            suggestion_refresh_found = configParser.getint('suggestion', 'suggestion_refresh_found')
        if configParser.has_option('suggestion', 'suggestion_refresh_notfound'):
            suggestion_refresh_notfound = configParser.getint('suggestion', 'suggestion_refresh_notfound')
    else:
        logger.critical('Config file not found (%s).' % conf_file)
        return False

    ciscosupport.set_logging(loglevel)

    sugestion_dir = base_path + '/suggestion'
    path_found = ciscosupport.expand_path(sugestion_dir + '/found/')
    path_not_found = ciscosupport.expand_path(sugestion_dir + '/not_found/')
    path_request = ciscosupport.expand_path(sugestion_dir + '/request/')

    # create list of PIDs to request software suggestion for
    pids = ciscosupport.get_ids_from_dir(path_request)
    logger.debug('pid requests : %s' % pids)
    # remove already known PIDs from list
    pids = ciscosupport.remove_ids_from_list(pids, path_found)
    logger.debug('pid requests : %s' % pids)
    # remove PIDs already requested with unknown software suggestion status from list
    pids = ciscosupport.remove_ids_from_list(pids, path_not_found)
    logger.debug('pid requests : %s' % pids)

    # refresh notfound PIDs after 7 days by default
    pids = ciscosupport.refresh_ids_from_dir(path_not_found, suggestion_refresh_notfound, pids, True)
    logger.debug('pid requests : %s' % pids)
    # refresh found PIDs after 31 days by default
    pids = ciscosupport.refresh_ids_from_dir(path_found, suggestion_refresh_found, pids, False)
    logger.debug('pid requests : %s' % pids)

    if pids == []:
        logger.debug('all list are empty. Do nothing.')
        return

    # wait random time after startup
    if wait_after_start:
        ciscosupport.sleep_random(max_wait_time)

    # get time since epoch in seconds
    starttime = int(time.time())

    # request access_token if no token available or if is expired
    if (access_token == '') or lifetime < starttime:
        response = ciscoapi.get_access_token()
        access_token = response.get('access_token', '')
        lifetime = int(response.get('lifetime', 3600))

    lifetime = starttime + lifetime - 30

    if access_token == '':
        logger.critical('failed to get access_token')
        return

    if pids != []:
        temp_pids = []
        for pid in pids:
            pid = pid.replace('_', '/')
            temp_pids.append(pid)
        pids = temp_pids

        # path_not_found = ciscosupport.get_cisco_path_not_found_by_pid(pids, access_token)
        suggestions = ciscoapi.get_suggested_release_by_pid(pids, access_token)

        # compact suggestions to by bid
        suggestions_by_pid, pids_notfound = compact_sugesstions_by_pid(suggestions)

        # save found suggestions
        pids = save_suggested(suggestions_by_pid, path_found)
        # delete requests for known pids
        ciscosupport.remove_ids_from_dir(pids, path_request)

        # save pids not found
        pids = save_pids_notfound(pids_notfound, path_not_found)
        ciscosupport.remove_ids_from_dir(pids, path_request)
        # delete pids from found were the status changed to notfound
        ciscosupport.remove_ids_from_dir(pids, path_found)


main()
